package pathgame.gameplay;

import java.util.Iterator;
import java.util.LinkedList;
import pathgame.tilemap.Tile;
import pathgame.tilemap.TileMap;

public class Player
{
    private static final float SPEED = 0.125f;

    private PlayerAbilities abilities;

    private float lastX = 0;
    private float lastY = 0;
    private float x = 0;
    private float y = 0;

    private float velX = 0;
    private float velY = 0;
    private boolean isMoving = false;
    private int currSpeedSlowdown = 1;

    private final int energySupply;
    private int energyUsed = 0;
    private final int objectivesAmount;
    private int objectivesVisited = 0;
    private final LinkedList<MinusStepsValues> steps = new LinkedList<>();
    //private int minusEnergy = 0;

    public Player(int energySupply, int objectivesAmount)
    {
        this.energySupply = energySupply;
        this.objectivesAmount = objectivesAmount;
        abilities = PlayerAbilities.NORMAL;
    }

    public float getLastX()
    {
        return lastX;
    }

    public float getLastY()
    {
        return lastY;
    }

    public float getX()
    {
        return x;
    }

    public float getY()
    {
        return y;
    }
    
    private void tickSteps()
    {
        Iterator<MinusStepsValues> iter = steps.iterator();
        while(iter.hasNext())
        {
            MinusStepsValues next = iter.next();
            if(next.tick())
            {
                iter.remove();
            }
        }
    }

    public void tick(TileMap map)
    {
        tickSteps();
        
        int currentTileX = Math.round(x);
        int currentTileY = Math.round(y);
        Tile currentTile = map.getTile(currentTileX, currentTileY);
        
        currSpeedSlowdown = currentTile.getEnergyCost(abilities);
        lastX = x;
        lastY = y;

        if(isOnTile())
        {
            velX = 0.0f;
            velY = 0.0f;
            if(isMoving)
            {
                currentTile.onEnter(this, map, currentTileX, currentTileY);
                ++objectivesVisited;//TODO check for objective
                //System.out.print(map.getTile((int)x,(int) y).getEnergyCost());
                steps.addLast(new MinusStepsValues(currSpeedSlowdown));
                energyUsed += currSpeedSlowdown;
            }
            isMoving = false;
        }

        if(Keys.LEFT_KEY.isDown() && !isMoving && x > 0 && !map.getTile(currentTileX - 1, currentTileY).isBlockingMovement())
        {
            velX = -SPEED;
            isMoving = true;
            currentTile.onLeave(this, map, currentTileX, currentTileY);
        }
        else if(Keys.RIGHT_KEY.isDown() && !isMoving && x < map.getWidth() - 1 && !map.getTile(currentTileX + 1, currentTileY).isBlockingMovement())
        {
            velX = SPEED;
            isMoving = true;
            currentTile.onLeave(this, map, currentTileX, currentTileY);
        }
        else if(Keys.UP_KEY.isDown() && !isMoving && y > 0 && !map.getTile(currentTileX, currentTileY - 1).isBlockingMovement())
        {
            velY = -SPEED;
            isMoving = true;
            currentTile.onLeave(this, map, currentTileX, currentTileY);
        }
        else if(Keys.DOWN_KEY.isDown() && !isMoving && y < map.getHeight() - 1 && !map.getTile(currentTileX, currentTileY + 1).isBlockingMovement())
        {
            velY = SPEED;
            isMoving = true;
            currentTile.onLeave(this, map, currentTileX, currentTileY);
        }

        float moveX = Math.abs(velX / currSpeedSlowdown);
        float moveY = Math.abs(velY / currSpeedSlowdown);
        if(velX < 0.0f)
        {
            float d = x - (float) Math.floor(x);
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveX = -Math.min(moveX, d);
        }
        else if(velX > 0.0f)
        {
            float d = (float) Math.ceil(x) - x;
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveX = Math.min(moveX, d);
        }
        else if(velY < 0.0f)
        {
            float d = y - (float) Math.floor(y);
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveY = -Math.min(moveY, d);
        }
        else if(velY > 0.0f)
        {
            float d = (float) Math.ceil(y) - y;
            if(d < 0.01f)
            {
                d = 1.0f;
            }
            moveY = Math.min(moveY, d);
        }

        //System.out.println(moveX);
        x += moveX;
        y += moveY;
    }

    private boolean isOnTile()
    {
        return Math.abs(x - Math.round(x)) < 0.01f && Math.abs(y - Math.round(y)) < 0.01f;
    }

    public float getVelX()
    {
        return velX;
    }

    public float getVelY()
    {
        return velY;
    }

    public int getEnergySupply()
    {
        return energySupply;
    }

    public int getEnergyLeft()
    {
        return energySupply - energyUsed;
    }

    public int getEnergyUsed()
    {
        return energyUsed;
    }

    public int getObjectivesAmount()
    {
        return objectivesAmount;
    }

    public int getObjectivesVisited()
    {
        return objectivesVisited;
    }

    public LinkedList<MinusStepsValues> getLastSteps()
    {
        return steps;
    }
    
    public void setAbilities(PlayerAbilities playerAbilities)
    {
        abilities = playerAbilities;
    }
    
    public PlayerAbilities getAbilities()
    {
        return abilities;
    }
}
